@@ -62,6 +62,7 @@ typedef enum {
 } DataFormat;
 
 typedef enum {
+	TRANSACTION_FORMAT_NONE      = 0,
 	TRANSACTION_FORMAT_DATA      = 1 << 0,
 	TRANSACTION_FORMAT_ONTOLOGY  = 1 << 1,
 } TransactionFormat;
@@ -111,6 +112,9 @@ static struct {
 
 static JournalReader reader = {0};
 static JournalWriter writer = {0};
+static JournalWriter ontology_writer = {0};
+
+static TransactionFormat current_transaction_format;
 
 #if GLIB_CHECK_VERSION (2, 24, 2)
 static gboolean tracker_db_journal_rotate (void);
@@ -549,6 +553,26 @@ tracker_db_journal_init (const gchar *filename,
 	return ret;
 }
 
+static gboolean
+db_journal_ontology_init (void)
+{
+	gboolean ret;
+	gchar *filename;
+
+	g_return_val_if_fail (ontology_writer.journal == 0, FALSE);
+
+	filename = g_build_filename (g_get_user_data_dir (),
+	                             "tracker",
+	                             "data",
+	                             TRACKER_DB_JOURNAL_ONTOLOGY_FILENAME,
+	                             NULL);
+
+	ret = db_journal_writer_init (&ontology_writer, FALSE, FALSE, filename);
+	g_free (filename);
+
+	return ret;
+}
+
 static gboolean
 db_journal_writer_shutdown (JournalWriter *jwriter)
 {
@@ -600,6 +624,9 @@ db_journal_writer_start_transaction (JournalWriter    *jwriter,
 	guint size;
 
 	g_return_val_if_fail (jwriter->journal > 0, FALSE);
+	g_return_val_if_fail (current_transaction_format == TRANSACTION_FORMAT_NONE, FALSE);
+
+	current_transaction_format = kind;
 
 	size = sizeof (guint32) * 3;
 	cur_block_maybe_expand (jwriter, size);
@@ -636,7 +663,11 @@ tracker_db_journal_start_transaction (time_t time)
 gboolean
 tracker_db_journal_start_ontology_transaction (time_t time)
 {
-	return db_journal_writer_start_transaction (&writer, time,
+	if (!db_journal_ontology_init ()) {
+		return FALSE;
+	}
+
+	return db_journal_writer_start_transaction (&ontology_writer, time,
 	                                            TRANSACTION_FORMAT_ONTOLOGY);
 }
 
@@ -688,6 +719,10 @@ tracker_db_journal_append_delete_statement (gint         g_id,
                                             gint         p_id,
                                             const gchar *object)
 {
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		return TRUE;
+	}
+
 	return db_journal_writer_append_delete_statement (&writer,
 	                                                  g_id, s_id, p_id, object);
 }
@@ -738,6 +773,10 @@ tracker_db_journal_append_delete_statement_id (gint g_id,
                                                gint p_id,
                                                gint o_id)
 {
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		return TRUE;
+	}
+
 	return db_journal_writer_append_delete_statement_id (&writer,
 	                                                     g_id, s_id, p_id, o_id);
 }
@@ -790,6 +829,10 @@ tracker_db_journal_append_insert_statement (gint         g_id,
                                             gint         p_id,
                                             const gchar *object)
 {
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		return TRUE;
+	}
+
 	return db_journal_writer_append_insert_statement (&writer,
 	                                                  g_id, s_id, p_id, object);
 }
@@ -840,6 +883,10 @@ tracker_db_journal_append_insert_statement_id (gint g_id,
                                                gint p_id,
                                                gint o_id)
 {
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		return TRUE;
+	}
+
 	return db_journal_writer_append_insert_statement_id (&writer,
 	                                                     g_id, s_id, p_id, o_id);
 }
@@ -875,16 +922,35 @@ gboolean
 tracker_db_journal_append_resource (gint         s_id,
                                     const gchar *uri)
 {
-	return db_journal_writer_append_resource (&writer, s_id, uri);
+	gboolean ret;
+
+	g_return_val_if_fail (current_transaction_format != TRANSACTION_FORMAT_NONE, FALSE);
+
+
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		ret = db_journal_writer_append_resource (&ontology_writer, s_id, uri);
+	} else {
+		ret = db_journal_writer_append_resource (&writer, s_id, uri);
+	}
+
+	return ret;
 }
 
 gboolean
 tracker_db_journal_rollback_transaction (void)
 {
 	g_return_val_if_fail (writer.journal > 0, FALSE);
+	g_return_val_if_fail (current_transaction_format != TRANSACTION_FORMAT_NONE, FALSE);
 
 	cur_block_kill (&writer);
 
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		cur_block_kill (&ontology_writer);
+		db_journal_writer_shutdown (&ontology_writer);
+	}
+
+	current_transaction_format = TRANSACTION_FORMAT_NONE;
+
 	return TRUE;
 }
 
@@ -949,18 +1015,27 @@ tracker_db_journal_commit_db_transaction (void)
 {
 	gboolean ret;
 
-	ret = db_journal_writer_commit_db_transaction (&writer);
+	g_return_val_if_fail (current_transaction_format != TRANSACTION_FORMAT_NONE, FALSE);
+
+	if (current_transaction_format == TRANSACTION_FORMAT_ONTOLOGY) {
+		ret = db_journal_writer_commit_db_transaction (&ontology_writer);
+		db_journal_writer_shutdown (&ontology_writer);
+	} else {
+		ret = db_journal_writer_commit_db_transaction (&writer);
 
 #if GLIB_CHECK_VERSION (2, 24, 2)
-	if (ret) {
-		if (rotating_settings.do_rotating && (writer.cur_size > rotating_settings.chunk_size)) {
-			if (!tracker_db_journal_rotate ()) {
-				g_critical ("Could not rotate journal, %s", g_strerror (errno));
-				ret = FALSE;
+		if (ret) {
+			if (rotating_settings.do_rotating && (writer.cur_size > rotating_settings.chunk_size)) {
+				if (!tracker_db_journal_rotate ()) {
+					g_critical ("Could not rotate journal, %s", g_strerror (errno));
+					ret = FALSE;
+				}
 			}
 		}
-	}
 #endif /* GLib check */
+	}
+
+	current_transaction_format = TRANSACTION_FORMAT_NONE;
 
 	return ret;
 }
@@ -1150,6 +1225,29 @@ tracker_db_journal_reader_init (const gchar *filename)
 	return db_journal_reader_init (&reader, TRUE, filename);
 }
 
+gboolean
+tracker_db_journal_reader_ontology_init (const gchar *filename) {
+	gchar *filename_used;
+	gboolean result;
+
+	/* Used mostly for testing */
+	if (G_UNLIKELY (filename)) {
+		filename_used = g_strdup (filename);
+	} else {
+		filename_used = g_build_filename (g_get_user_data_dir (),
+		                                  "tracker",
+		                                  "data",
+		                                  TRACKER_DB_JOURNAL_ONTOLOGY_FILENAME,
+		                                  NULL);
+	}
+
+	result = tracker_db_journal_reader_init (filename_used);
+
+	g_free (filename_used);
+
+	return result;
+}
+
 gsize
 tracker_db_journal_reader_get_size_of_correct (void)
 {
@@ -1556,18 +1654,23 @@ tracker_db_journal_reader_verify_last (const gchar  *filename,
 	JournalReader jreader = { 0 };
 
 	if (db_journal_reader_init (&jreader, FALSE, filename)) {
-		entry_size_check = read_uint32 (jreader.end - 4);
 
-		if (jreader.end - entry_size_check < jreader.current) {
-			g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
-			             "Damaged journal entry at end of journal");
+		if (jreader.end != jreader.current) {
+			entry_size_check = read_uint32 (jreader.end - 4);
+
+			if (jreader.end - entry_size_check < jreader.current) {
+				g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
+				             "Damaged journal entry at end of journal");
+				db_journal_reader_shutdown (&jreader);
+				return FALSE;
+			}
+
+			jreader.current = jreader.end - entry_size_check;
+			success = db_journal_reader_next (&jreader, FALSE, NULL);
 			db_journal_reader_shutdown (&jreader);
-			return FALSE;
+		} else {
+			success = TRUE;
 		}
-
-		jreader.current = jreader.end - entry_size_check;
-		success = db_journal_reader_next (&jreader, FALSE, NULL);
-		db_journal_reader_shutdown (&jreader);
 	}
 
 	return success;
